# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
# Generated file, DO NOT EDIT
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------------------------

from .models import *
from .test_results_client import TestResultsClient

__all__ = [
    'AggregatedDataForResultTrend',
    'AggregatedResultDetailsByOutcome',
    'AggregatedResultsAnalysis',
    'AggregatedResultsByOutcome',
    'AggregatedResultsDifference',
    'AggregatedRunsByOutcome',
    'AggregatedRunsByState',
    'BuildConfiguration',
    'BuildCoverage',
    'BuildReference',
    'CodeCoverageData',
    'CodeCoverageStatistics',
    'CodeCoverageSummary',
    'CoverageStatistics',
    'CustomTestField',
    'DtlEnvironmentDetails',
    'FailingSince',
    'FieldDetailsForTestResults',
    'FileCoverageRequest',
    'FlakyDetection',
    'FlakyDetectionPipelines',
    'FlakySettings',
    'FunctionCoverage',
    'GraphSubjectBase',
    'IdentityRef',
    'JobReference',
    'ModuleCoverage',
    'NewTestResultLoggingSettings',
    'PhaseReference',
    'PipelineReference',
    'PipelineTestMetrics',
    'QueryModel',
    'ReferenceLinks',
    'ReleaseReference',
    'ResultsAnalysis',
    'ResultsFilter',
    'ResultsSummaryByOutcome',
    'ResultSummary',
    'RunCreateModel',
    'RunFilter',
    'RunStatistic',
    'RunSummary',
    'RunSummaryModel',
    'RunUpdateModel',
    'ShallowReference',
    'ShallowTestCaseResult',
    'SharedStepModel',
    'StageReference',
    'TeamProjectReference',
    'TestActionResultModel',
    'TestAttachment',
    'TestAttachmentReference',
    'TestAttachmentRequestModel',
    'TestCaseResult',
    'TestCaseResultAttachmentModel',
    'TestCaseResultIdentifier',
    'TestEnvironment',
    'TestFailureDetails',
    'TestFailuresAnalysis',
    'TestFlakyIdentifier',
    'TestHistoryQuery',
    'TestIterationDetailsModel',
    'TestLog',
    'TestLogReference',
    'TestLogStoreEndpointDetails',
    'TestMessageLogDetails',
    'TestMethod',
    'TestOperationReference',
    'TestResolutionState',
    'TestResultDocument',
    'TestResultFailuresAnalysis',
    'TestResultHistory',
    'TestResultHistoryDetailsForGroup',
    'TestResultHistoryForGroup',
    'TestResultMetaData',
    'TestResultMetaDataUpdateInput',
    'TestResultModelBase',
    'TestResultParameterModel',
    'TestResultPayload',
    'TestResultsContext',
    'TestResultsDetails',
    'TestResultsDetailsForGroup',
    'TestResultsQuery',
    'TestResultsSettings',
    'TestResultSummary',
    'TestResultsUpdateSettings',
    'TestResultTrendFilter',
    'TestRun',
    'TestRunCoverage',
    'TestRunStatistic',
    'TestSettings',
    'TestSubResult',
    'TestSummaryForWorkItem',
    'TestTag',
    'TestTagSummary',
    'TestTagsUpdateModel',
    'TestToWorkItemLinks',
    'WorkItemReference',
    'WorkItemToTestLinks',
    'TestResultsClient'
]
